/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 1999 - 2006, Digium, Inc.
 *
 * Mark Spencer <markster@digium.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \Brief App to lookup the callerid number, and see if it is in the specified database list
 *
 * \ingroup applications
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 7221 $")

#include "asterisk/lock.h"
#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/options.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/translate.h"
#include "asterisk/image.h"
#include "asterisk/callerid.h"
#include "asterisk/astdb.h"
#include "asterisk/options.h"
#include "asterisk/app.h" 

static char *tdesc = "Look up Caller*ID name/number from the specified database list";

static char *app = "LookupList";

static char *synopsis = "Look up Caller*ID name/number from the specified database list";

static char *descrip =
  "  LookupList(database_family,options): Looks up the Caller*ID number on the active\n"
  "channel in the Asterisk database for the specified database family.  \n"
  "The option string may contain the following character:\n"
  "	'j' -- jump to n+101 priority if the number/name is found in the database list\n"
  "This application sets the following channel variable upon completion:\n"
  "	LOOKUPLISTSTATUS		The status of the List lookup as a text string, one of\n"
  "		FOUND | NOTFOUND\n"
  "Example: exten => 1234,1,LookupList(friendlist)\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int
lookuplist_exec (struct ast_channel *chan, void *data)
{
	char *parse = NULL;
	char list[1];
	struct localuser *u;
	int bl = 0;
	int priority_jump = 0;
 	AST_DECLARE_APP_ARGS(args,
                 AST_APP_ARG(d);
		 AST_APP_ARG(o);
	);

	LOCAL_USER_ADD(u);

 	if (ast_strlen_zero(data)) {
                 ast_log(LOG_WARNING, "LookupList requires an argument (database_family)\n");
		                 return 0;
	}

	parse = ast_strdupa(data);
	if (!parse) {
	 	  ast_log(LOG_ERROR, "Out of memory!\n");
	  	  LOCAL_USER_REMOVE(u);
		  return -1;
	}
	AST_STANDARD_APP_ARGS(args, parse);

 	if (args.o) {
                 if (strchr(args.o, 'j'))
		 priority_jump = 1;
	}		

	if (chan->cid.cid_num) {
		if (!ast_db_get(args.d, chan->cid.cid_num, list, sizeof (list))) {
			if (option_verbose > 2)
				ast_log(LOG_NOTICE, "Database %s number %s found\n",args.d, chan->cid.cid_num);
			bl = 1;
		}
	}
	if (chan->cid.cid_name) {
		if (!ast_db_get(args.d, chan->cid.cid_name, list, sizeof (list))) {
			if (option_verbose > 2)
				ast_log (LOG_NOTICE, "Database %s name \"%s\" found\n",args.d, chan->cid.cid_name);
			bl = 1;
		}
	}

	if (bl) {
		if (priority_jump || option_priority_jumping) 
			ast_goto_if_exists(chan, chan->context, chan->exten, chan->priority + 101);
		pbx_builtin_setvar_helper(chan, "LOOKUPLISTSTATUS", "FOUND");
	} else
		pbx_builtin_setvar_helper(chan, "LOOKUPLISTSTATUS", "NOTFOUND");	

	LOCAL_USER_REMOVE(u);

	return 0;
}

int unload_module (void)
{
	int res;

	res = ast_unregister_application(app);

	STANDARD_HANGUP_LOCALUSERS;

	return res;	
}

int load_module (void)
{
	return ast_register_application (app, lookuplist_exec, synopsis,descrip);
}

char *description (void)
{
	return tdesc;
}

int usecount (void)
{
	int res;
	STANDARD_USECOUNT (res);
	return res;
}

char *key ()
{
	return ASTERISK_GPL_KEY;

}
